@@ -5,7 +5,7 @@
 ;;   Copyright (C) 2002,2003,2004,2005 Kiyoka Nishyama
 ;;   This program was derived from yc.el-4.0.13(auther: knak)
 ;;
-;;     $Date: 2005/08/28 09:59:18 $
+;;     $Date: 2005/09/21 14:12:15 $
 ;;
 ;; This file is part of Sumibi
 ;;
@@ -49,7 +49,7 @@
   :group 'input-method
   :group 'Japanese)
 
-(defcustom sumibi-server-url "https://sumibi.org/cgi-bin/sumibi/stable/sumibi.cgi"
+(defcustom sumibi-server-url "https://sumibi.org/cgi-bin/sumibi/testing/sumibi.cgi"
   "Sumibi$B%5!<%P!<$N(BURL$B$r;XDj$9$k!#(B"
   :type  'string
   :group 'sumibi)
@@ -110,6 +110,11 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   :type  'string
   :group 'sumibi)
 
+(defcustom sumibi-use-viper nil
+  "*Non-nil $B$G$"$l$P!"(BVIPER $B$KBP1~$9$k!#(B"
+  :type 'boolean
+  :group 'sumibi)
+
 
 (defvar sumibi-mode nil             "$B4A;zJQ49%H%0%kJQ?t(B")
 (defvar sumibi-mode-line-string     " Sumibi")
@@ -133,6 +138,13 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		  minor-mode-map-alist)))
 
 
+;;;
+;;; hooks
+;;;
+(defvar sumibi-mode-hook nil)
+(defvar sumibi-select-mode-hook nil)
+(defvar sumibi-select-mode-end-hook nil)
+
 
 ;;--- $B%G%P%C%0%a%C%;!<%8=PNO(B
 (defvar sumibi-debug nil)		; $B%G%P%C%0%U%i%0(B
@@ -342,7 +354,8 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 	    (sumibi-trap-server-down
 	     (beep)
 	     (message (error-message-string err))
-	     (setq sumibi-select-mode nil)) )
+	     (setq sumibi-select-mode nil))
+	    (run-hooks 'sumibi-select-mode-end-hook))
 	nil))))
 
 
@@ -530,6 +543,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   ;; $B8uJdHV9f%j%9%H$r%P%C%/%"%C%W$9$k!#(B
   (setq sumibi-cand-n-backup (copy-list sumibi-cand-n))
   (setq sumibi-select-mode nil)
+  (run-hooks 'sumibi-select-mode-end-hook)
   (sumibi-select-update-display))
 
 ;; $B8uJdA*Br$r%-%c%s%;%k$9$k(B
@@ -539,6 +553,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   ;; $B%+%l%s%H8uJdHV9f$r%P%C%/%"%C%W$7$F$$$?8uJdHV9f$GI|85$9$k!#(B
   (setq sumibi-cand-n (copy-list sumibi-cand-n-backup))
   (setq sumibi-select-mode nil)
+  (run-hooks 'sumibi-select-mode-end-hook)
   (sumibi-select-update-display))
 
 ;; $BA0$N8uJd$K?J$a$k(B
@@ -684,6 +699,7 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 	(let
 	    ((cnt 0))
 	  (setq sumibi-select-mode t)
+	  (run-hooks 'sumibi-select-mode-hook)
 	  (setq sumibi-cand 0)		; $BJ8@aHV9f=i4|2=(B
 	  
 	  (sumibi-debug-print "henkan mode ON\n")
@@ -750,6 +766,36 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 
       result)))
   
+;;;
+;;; with viper
+;;;
+;; code from skk-viper.el
+(defun sumibi-viper-normalize-map ()
+  (let ((other-buffer
+	 (if (featurep 'xemacs)
+	     (local-variable-p 'minor-mode-map-alist nil t)
+	   (local-variable-if-set-p 'minor-mode-map-alist))))
+    ;; for current buffer and buffers to be created in the future.
+    ;; substantially the same job as viper-harness-minor-mode does.
+    (viper-normalize-minor-mode-map-alist)
+    (setq-default minor-mode-map-alist minor-mode-map-alist)
+    (when other-buffer
+      ;; for buffers which are already created and have
+      ;; the minor-mode-map-alist localized by Viper.
+      (dolist (buf (buffer-list))
+	(with-current-buffer buf
+	  (unless (assq 'sumibi-mode minor-mode-map-alist)
+	    (setq minor-mode-map-alist
+		  (append (list (cons 'sumibi-mode sumibi-mode-map)
+				(cons 'sumibi-select-mode
+				      sumibi-select-mode-map))
+			  minor-mode-map-alist)))
+	  (viper-normalize-minor-mode-map-alist))))))
+
+(defun sumibi-viper-init-function ()
+  (sumibi-viper-normalize-map)
+  (remove-hook 'sumibi-mode-hook 'sumibi-viper-init-function))
+
 
 ;;;
 ;;; human interface
@@ -804,9 +850,14 @@ point $B$+$i9TF,J}8~$KF1<o$NJ8;zNs$,B3$/4V$r4A;zJQ49$7$^$9!#(B
       (make-local-variable 'sumibi-mode))
   (if global
       (progn
-	(setq-default sumibi-mode t)
+	(setq-default sumibi-mode (if (null arg) (not sumibi-mode)
+				    (> (prefix-numeric-value arg) 0)))
 	(sumibi-kill-sumibi-mode))
-    (setq sumibi-mode t)))
+    (setq sumibi-mode (if (null arg) (not sumibi-mode)
+			(> (prefix-numeric-value arg) 0))))
+  (when sumibi-use-viper
+    (add-hook 'sumibi-mode-hook 'sumibi-viper-init-function))
+  (when sumibi-mode (run-hooks 'sumibi-mode-hook)))
 
 
 ;; buffer local $B$J(B sumibi-mode $B$r:o=|$9$k4X?t(B
@@ -842,13 +893,10 @@ point $B$+$i9TF,J}8~$KF1<o$NJ8;zNs$,B3$/4V$r4A;zJQ49$7$^$9!#(B
 (setq default-input-method "japanese-sumibi")
 
 (defconst sumibi-version
-"0.4.0"
+"0.5.0"
   )
 (defun sumibi-version (&optional arg)
   "$BF~NO%b!<%IJQ99(B"
   (interactive "P")
   (message sumibi-version))
 (provide 'sumibi)
-
-;; Sumibi $B%b!<%I$r%P%C%U%!A4BN$GM-8z$K$9$k(B
-(global-sumibi-mode 1)
